import { Injectable, Inject } from '@angular/core';
import uuidv4 from 'uuid/v4';
import { UserEntity } from '../../entities/user.entity';
import { AUTH_OPTIONS_TOKEN } from '../auth/auth.options.token';
import { AuthConfig } from '../auth/entities/auth.config';

// const APP_PREFIX: string = 'att-front-app';
let APP_PREFIX: string = 'att-front-app';
let USER_KEY: string = `${APP_PREFIX}-profile`;
let ACCESS_TOKEN_KEY: string = `${APP_PREFIX}-access_token`;
let REFRESH_TOKEN_KEY: string = `${APP_PREFIX}-refresh_token`;
const DEVICE_ID: string = `${APP_PREFIX}-device-id`;

const TIME_REFRESH_KEY: string = 'time-for-refresh';
const TOT_TIME_KEY: string = 'tot-time';

@Injectable()
export class StorageService {
  config: AuthConfig;

  constructor(@Inject(AUTH_OPTIONS_TOKEN)  config: AuthConfig ) {
    this.config = config;
    if (this.config.appPrefix !== '') {
      APP_PREFIX = this.config.appPrefix;
      USER_KEY = USER_KEY.replace('att-front-app', APP_PREFIX);
      ACCESS_TOKEN_KEY = ACCESS_TOKEN_KEY.replace('att-front-app', APP_PREFIX);
      REFRESH_TOKEN_KEY = REFRESH_TOKEN_KEY.replace('att-front-app', APP_PREFIX);
    }
  }
  

  isSupported(): boolean {

    const dummy = 'dummy';
    try {
      localStorage.setItem(dummy, dummy);
      localStorage.removeItem(dummy);
      return true;
    } catch(e) {
      return false;
    }

  }

  get(key: string): string {
    return localStorage.getItem(key);
  }

  set(key: string, data: string) {
    localStorage.setItem(key, data);
  }

  getItem<T>(key: string): T {

    try {
      const value: string = localStorage.getItem(key);
      return JSON.parse(value) as T;
    }
    catch(e) {
      return undefined;
    }

  }

  setItem<T>(key: string, data: T): boolean {
    try {
      localStorage.setItem(key, JSON.stringify(data));
      return true;
    }
    catch(e) {
      return false;
    }
  }

  getAccessToken(): string {
    return localStorage.getItem(ACCESS_TOKEN_KEY);
  }

  getRefreshToken(): string {
    return localStorage.getItem(REFRESH_TOKEN_KEY)
  }

  setAccessToken(token: string): void {
    localStorage.setItem(ACCESS_TOKEN_KEY, token);
  }

  setRefreshToken(token: string) : void {
    localStorage.setItem(REFRESH_TOKEN_KEY, token);
  }

  setUser(user: UserEntity): void {
    localStorage.setItem(USER_KEY, JSON.stringify(user));
  }

  getUser(): UserEntity {
    return JSON.parse(localStorage.getItem(USER_KEY));
  }

  setRefreshTime(time: number): void {
    localStorage.setItem(TIME_REFRESH_KEY, JSON.stringify(time));
  }

  getRefreshTime(): number {
    return JSON.parse(localStorage.getItem(TIME_REFRESH_KEY));
  }

  setTotTime(time: number): void {
    localStorage.setItem(TOT_TIME_KEY, JSON.stringify(time));
  }

  getTotTime(): number {
    return JSON.parse(localStorage.getItem(TOT_TIME_KEY));
  }

  resetStorage(): void {
    localStorage.clear();
  }

  resetUser(): void {
    localStorage.removeItem(USER_KEY);
  }

  resetAccessToken(): void {
    localStorage.removeItem(ACCESS_TOKEN_KEY);
  }

  resetRefresToken(): void {
    localStorage.removeItem(REFRESH_TOKEN_KEY)
  }

  setDeviceId(deviceId: string): void {
    localStorage.setItem(DEVICE_ID, deviceId);
  }

  getDeviceId(): string {
    try {
      if (localStorage.getItem(DEVICE_ID) === null) {
        const uuid = uuidv4();
        const uuidBase64 = btoa(uuid);
  
        console.log('uuid', uuid);
        console.log('base64', uuidBase64);
  
        localStorage.setItem(DEVICE_ID, uuidBase64);
        return uuidBase64;
      }
  
      return localStorage.getItem(DEVICE_ID);
    }
    catch(e) {
      return '';
    }
  }

  resetDeviceId(): void {
    localStorage.removeItem(DEVICE_ID);
  }
}
