import { ModuleWithProviders, NgModule } from '@angular/core';
import { AuthOptions } from './services/auth/auth.options';
import { AUTH_OPTIONS_TOKEN } from './services/auth/auth.options.token';
import { HttpClient, HttpClientModule} from '@angular/common/http';
import { JWT_OPTIONS, JwtModule } from '@auth0/angular-jwt';
import { StorageService } from './services/storage/storage.service';
import { AuthService } from './services/auth/auth.service';
import { ToasterModule } from 'angular2-toaster';
import {NotificationService} from './services/notification/notification.service';

const getDomains = (url: string | string[]): string[] => {

  if (typeof url === 'string') {
    return [new URL(url).host];
  } else if (<string[]> url) {
    return url.map(item => {
      return new URL(item).host
    })
  }

};

export function jwtOptionsFactory(storage: StorageService, auth: AuthService) {

  if (auth.configuration && auth.configuration.authEndpoint) {

    const defaultBlacklisted = `${auth.configuration.authEndpoint}/oauth/token`;
    const blacklisted: any[] = [defaultBlacklisted];

    if(auth.configuration.excludedEndpoints) {
      const processed = auth.configuration.excludedEndpoints.map(item => {
        return item.regex ? new RegExp(item.value) : item.value;
      });
      blacklisted.push(...processed);
    }

    return {
      whitelistedDomains: getDomains(auth.configuration.servicesEndpoint),
      blacklistedRoutes: blacklisted,
      tokenGetter: () => {
        return storage.getAccessToken();
      }
    }
  }

  return {
    whitelistedDomains: [],
    blacklistedRoutes:[],
    tokenGetter: () => {
      return storage.getAccessToken();
    }
  }
}

@NgModule({
  imports: [
    HttpClientModule,
    ToasterModule.forRoot(),
    JwtModule.forRoot({
      jwtOptionsProvider: {
        provide: JWT_OPTIONS,
        useFactory: jwtOptionsFactory,
        deps: [StorageService, AuthService]
      }
    })
  ],
  declarations: [],
  exports: [],
  providers: [
    HttpClient,
    NotificationService
  ]
})
export class TcpipModule {
  static forRoot(options: AuthOptions): ModuleWithProviders {
    return {
      ngModule: TcpipModule,
      providers: [
        options.optionsProvider ||
        {
          provide: AUTH_OPTIONS_TOKEN,
          useValue: options.config
        }
      ]
    };
  }
}
